#!/usr/local/bin/perl
#
# PPMILTER ver 0.75 --- "Pigeon Post" SPECIAL MILTER
#          ( 1999.07.02 named by Masato MINDA <minmin@wide.ad.jp> )
# Copyright (c)1999-2002 by Kazunori ANDO <ando@kk.iij4u.or.jp> all rights reserved.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

use ExtUtils::testlib;
use Sendmail::Milter;
use Socket;

my %e = ();
my %rcpts = ();
my $CHKSUMTYPE  = 'SHA1';
my $debug = 1;

use vars qw(%vbody %hbody);
require "/usr/local/ppml/ppvirus.ph";
require "/usr/local/ppml/pphates.ph";

my %ppmilter_callbacks =
    (
     'connect' =>    \&pp_connect,
     'helo' =>       \&pp_helo,
     'envfrom' =>    \&pp_envfrom,
     'envrcpt' =>    \&pp_envrcpt,
     'header' =>     \&pp_header,
     'eoh' =>        \&pp_eoh,
     'body' =>       \&pp_body,
     'eom' =>        \&pp_eom,
     'abort' =>      \&pp_abort,
     'close' =>      \&pp_close,
     );

sub pp_connect{
    my $ctx = shift;
    my $hostname = shift;
    my $sockaddr_in = shift;
    my ($port, $iaddr);
    %e = ();

    $e{$ctx}{'connect_from:hostname'} = $hostname;
    
    if (defined $sockaddr_in)
    {
	($port, $iaddr) = sockaddr_in($sockaddr_in);
	$e{$ctx}{'connect_from:port'} = $port;
	$e{$ctx}{'connect_from:ip-addr'} = inet_ntoa($iaddr);
    }

    if ($debug) {
	print "*** pp_connect\n";
	foreach my $k (sort keys %{$e{$ctx}}){
	    print "$k => $e{$ctx}{$k}\n";
	}
    }

    if (($hostname eq "localhost")||($hostname eq "127.0.0.1")){
	$debug = 0;
    }

    return SMFIS_CONTINUE;
}

sub pp_helo{
    my $ctx = shift;
    my $helohost = shift;
    
    $e{$ctx}{'helohost'} = $helohost;

    if ($debug) {
	print "*** pp_helo\n";
	foreach my $k (sort keys %{$e{$ctx}}){
	    print "$k => $e{$ctx}{$k}\n";
	}
    }

    return SMFIS_CONTINUE;
}

sub pp_envfrom{
    my $ctx = shift;
    my $sender = shift;
    my @args = @_;
    my $message = "";

    $e{$ctx}{'sender'} = $sender;
    $e{$ctx}{'sender:opts'} = join(', ', @args);

    $ctx->setpriv(\$message);

    if ($debug) {
	print "*** pp_envfrom\n";
	foreach my $k (sort keys %{$e{$ctx}}){
	    print "$k => $e{$ctx}{$k}\n";
	}
    }

    return SMFIS_CONTINUE;
}

sub pp_envrcpt{
    my $ctx = shift;
    my @args = @_;

    $rcpts{$ctx}{$args[0]} = join(', ', @args);
    
    if ($debug) {
	print "*** pp_envrcpt\n";
	foreach my $k (sort keys %{$rcpts{$ctx}}){
	    print "$k => $rcpts{$ctx}{$k}\n";
	}
    }
    return SMFIS_CONTINUE;
}

sub pp_header{
    my $ctx = shift;
    my $headerf = shift;
    my $headerv = shift;
    my ($h,$c,$horg,$hc);

    $h = $headerf;
    $c = $headerv;
    $e{$ctx}{'Header'} .= "$h: $c\n";
    # field-name covert to unified form
    $horg = $h;
    $h =~ tr/A-Z/a-z/;
    # message-id -> Message-id
    if ( $h =~ /^(.)(.*)/ ){
	$hc = $1;
	$hc =~ tr/a-z/A-Z/;
	$h = $hc.$2;
    }
    # Message-id -> Message-Id
    while ( $h =~ /^(.+)(\-[a-z])(.+)/ ){
	$hc = $2;
	$hc =~ tr/a-z/A-Z/;
	$h = $1.$hc.$3;
    }
    $e{$ctx}{"OH:$h"} = $horg;
    if ($h =~ /Received/){
	$recn++;
    }
    if ((length($e{$ctx}{"H:$h"}) > 0)&&(length($c) > 0)){
	$e{$ctx}{"H:$h"} .= "$h: $c\n";
    }elsif((length($h) > 0) && (length($c) > 0)){ 
	$e{$ctx}{"H:$h"} .= "$c\n";
    }

    return SMFIS_CONTINUE;
}

sub pp_eoh{
    my $ctx = shift;

    if ($debug) {
	print "*** pp_eoh\n";
	foreach my $k (sort keys %{$e{$ctx}}){
	    print "$k => $e{$ctx}{$k}\n";
	}
    }

    return SMFIS_CONTINUE;
}

sub pp_body{
    my $ctx = shift;
    my $body_chunk = shift;
    my $len = shift;
    my $message_ref = $ctx->getpriv();
    
    ${$message_ref} .= $body_chunk;

    $ctx->setpriv($message_ref);

    return SMFIS_CONTINUE;
}

sub pp_eom{
    my $ctx = shift;
    my $message_ref = $ctx->getpriv();
    my $chunk;

    $e{$ctx}{'Body'} = ${$message_ref};

    $e{$ctx}{'size:body'} = length($e{$ctx}{'Body'});
    $e{$ctx}{'size:header'} = length($e{$ctx}{'Header'});

    if ($CHKSUMTYPE eq 'SHA1'){
	use Digest::SHA1 qw(sha1_base64);
	$e{$ctx}{'checksum:body'} = sha1_base64($e{$ctx}{'Body'});
    }elsif($CHKSUMTYPE eq 'MD5'){
	use Digest::MD5  qw(md5_base64);
	$e{$ctx}{'checksum:body'} = md5_base64($e{$ctx}{'Body'});
    }else{
	$e{$ctx}{'checksum:body'} = sprintf("%011d-%011d",$e{$ctx}{'size:body'},unpack("%31C*",$e{$ctx}{'Body'})); 
    }
    $e{$ctx}{'size'} = $e{$ctx}{'size:header'} + $e{$ctx}{'size:body'};

    if ($debug) {
	print "*** pp_eom\n";
	foreach my $k (sort keys %{$e{$ctx}}){
	    print "$k => $e{$ctx}{$k}\n";
	}
    }


    #### FUNCTION reject
    if (&CheckForReject($ctx)){
	$ctx->setpriv(undef);
	print "!!!!REJECTED(CFR)!!!! : $sig\n";
	return SMFIS_REJECT;
    }
    #### VIRUS reject
    foreach my $sig (keys %vbody){
	next if (length($sig) < 4);
	if ($e{$ctx}{'checksum:body'} eq $sig){
	    $ctx->setpriv(undef);
	    print "!!!!REJECTED(SIG)!!!! : $sig\n";
	    return SMFIS_REJECT;
	}
    }
    #### PATTERN reject
    foreach my $pat (keys %hbody){
	next if (length($pat) < 4);
	if (index($e{$ctx}{'Body'},$pat) != -1){
	    $ctx->setpriv(undef);
	    print "!!!!REJECTED(PAT)!!!! : $pat\n";
	    return SMFIS_REJECT;
	}
    }


    $ctx->setpriv(undef);
    return SMFIS_CONTINUE;
}

sub pp_abort{
    my $ctx = shift;

    if ($debug) {
	print "*** pp_abort\n";
	foreach my $k (sort keys %{$e{$ctx}}){
	    print "$k => $e{$ctx}{$k}\n";
	}
    }

    $ctx->setpriv(undef);
    delete $e{$ctx};

    return SMFIS_CONTINUE;
}

sub pp_close{
    my $ctx = shift;

    delete $e{$ctx};
    return SMFIS_CONTINUE;
}

BEGIN:
{
    # Get myfilter's connection information
    # from /etc/mail/sendmail.cf

    Sendmail::Milter::auto_setconn("ppmilter");
    Sendmail::Milter::register("ppmilter",\%ppmilter_callbacks, SMFI_CURR_ACTS);
    Sendmail::Milter::main();
      
    # Never reaches here, callbacks are called from Milter.
}
